options ps=50 ls=75 pageno=1;
goptions reset=all border ftext=swiss gunit=cm htext=0.4 htitle=0.5;


*********************************************************************;
**                                                                 **;
** AUTHOR: Chris Bilder                                            **;
** COURSE: STAT 4091                                               **;
** DATE: 8-28-02                                                   **;
** UPDATE: 12-23-02                                                **;
** PURPOSE: Show how to use functions in a SAS data step           **;
** NOTES:                                                          **;
**                                                                 **;
*********************************************************************;


*Will appear as the first line of every page in the output window;
title1 'Chris Bilder, STAT 4091';



**************************************************************************;
* Time and date example                                                  *;
**************************************************************************;

*Find the date and time;
data time;
  time_func = time();
  time = hour(time());
  minute = minute(time());
  second = second(time());
  month = month(today());
  day = day(today());
  year = year(today());
run;

title2 'The time is:';
proc print data=time;
run;




**************************************************************************;
* Max and sum examples                                                   *;
**************************************************************************;

data set1;
  input x1 x2 x3;
  datalines;
  1 2 3
  4 5 6
  ;
run;


data set2;
  set set1;
  sum1 = sum(x1, x2, x3);
  sum2 = sum(of x1-x3);
  max1 = max(x1, x2, x3);
  max2 = max(of x1-x3);
run;


title2 'Illustrate the sum and max functions';
proc print data=set2;
run;



**************************************************************************;
* Find percentiles from probability distribution                         *;
**************************************************************************;


*Find the percentiles;
*  Notice that you can also put functions in the same data step as where;
*  the data is read in!;
data set1;
  input alpha df1 df2;
  norm = probit(1-alpha);
  t = tinv(1-alpha, df1);
  chisq = cinv(1-alpha, df1);
  f = finv(1-alpha, df1, df2);
  datalines;
  0.01  5 5 
  0.025 5 5
  0.05  5 5
  0.10  5 5
  ;
run;
 

title2 'Percentiles from probability distributions';
proc print data=set1;
run;


*Another example;
data set1;
  input area_to_left;
  norm = probit(area_to_left);
  datalines;
  0.975
  0.025
  ;
run;
 

title2 'Percentiles from probability distributions';
proc print data=set1;
run;



**************************************************************************;
* Find probabilities from probability distributions                      *;
**************************************************************************;

*Find the probabilities;
* Remember that these are areas to the left of the number given! ;
* The numbers (percentiles) given are the same ones found in the data step for the;
* percentiles;  
data set1;
  input norm t chisq f df1 df2;
  pnorm = probnorm(norm);
  tprob = probt(t, df1);
  chisqprob = probchi(chisq, df1);
  fprob = probf(f, df1, df2);

  *Note: cdf('dist',quantile,parm-1,...,parm-k) also works;
  pnorm2 = cdf('normal', norm);
  datalines;
  2.32635 3.36493 15.0863 10.9670 5 5 
  1.95996 2.57058 12.8325 7.1464 5 5
  1.64485 2.01505 11.0705 5.0503 5 5
  1.28155 1.47588  9.2364 3.4530 5 5
  ;
run;
 

title2 'Probabilities from probability distributions';
proc print data=set1;
run;



*****************************************************************************;
* Find the p-value for the Section 4 cereal_more_summary.sas t-test example *;
*****************************************************************************;

data sugar_p;
  input t df;
  tprob = 2*(1-probt(abs(t), df));
  datalines;
  12.24 39
  ;
run;
 

title2 'The p-value is:';
proc print data=sugar_p;
run;



*****************************************************************************;
* Generating an observation from a probability distribution                 *;
*****************************************************************************;

*Simulate from a standard normal;
data normal1;
  *Let SAS choose the seed;
  norm = normal(0);
run;

data normal2;
  *I choose the seed;
  norm2 = rannor(4519);
run;

data normal3;
  CALL STREAMINIT(4519); 
  norm3 = rand('NORMAL'); 
run;


title2 'Simulated value from a normal distribution - 1';
proc print data=normal1;
run;

title2 'Simulated value from a normal distribution - 2';
proc print data=normal2;
run;

title2 'Simulated value from a normal distribution - 3';
proc print data=normal3;
run;


*****************************************************************************;
* Generating many observations from a probability distribution and then     *;
* construct a histogram                                                     *;
*****************************************************************************;

*Simulate from a standard normal;
data normal;
  do i = 1 to 1000;
    norm = normal(0);
	output;
  end; 
run;


title2 'First 10 simulated values from a normal distribution';
proc print data=normal;
  where i<=10;
run;


title2 'Histogram of data simulated from a normal with mu=0 and sigma=1';
proc univariate data=normal;
  var norm;
  histogram / normal(fill mu=0 sigma=1) cfill=yellow lgrid=34 midpoints=-5 to 5 by 0.5 ;
run;





*Simulate from a uniform(0,1);
data unif;
  do i = 1 to 1000;
    unif = ranuni(0);
	output;
  end; 
run;


title2 'First 10 simulated values from a uniform(0,1) distribution';
proc print data=unif;
  where i<=10;
run;

/*
title2 'Histogram of data simulated from a uniform(0,1) distribution';
proc univariate data=unif;
  var unif;
  histogram / normal(fill) cfill=yellow lgrid=34 midpoints=-0.55 to 1.55 by 0.1 ;

  *Note a beta distribution with parameters alpha=1 and beta=1 is also an uniform(0,1) distribution;
  histogram / beta(fill alpha=1 beta=1) cfill=yellow lgrid=34 midpoints=-0.55 to 1.55 by 0.1 ;
run;
*/


/*
**************************************************************************;
* Make a sound                                                           *;
**************************************************************************;

data _null_;
   *sound(frequency, duration in milliseconds);  
   call sound(523,2000);
run;



**************************************************************************;
* Send e-mail                                                            *;
**************************************************************************;

filename message email 'bilder@okstate.edu'
  subject='Hello STAT 4091 students';
  
data _null_;
  file message;
  put 'The final exam is schedule for ...';
  put '       ';
  put 'Go Big Red!';
run;
*/